/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package biz.nellemann.libcvrapi;

import java.io.IOException;

import biz.nellemann.libcvrapi.pojo.Company;
import com.google.gson.Gson;

import com.google.gson.JsonSyntaxException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import okhttp3.Credentials;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class CvrApi {

    private final static Logger log = LoggerFactory.getLogger(CvrApi.class);
    private final OkHttpClient client;

    private final String userAgent;
    private final String authenticationToken;

    protected String baseUrl = "https://rest.cvrapi.dk/v1/dk";


    public CvrApi(String userAgent, String authenticationToken) {
        this.userAgent = userAgent;
        this.authenticationToken = authenticationToken;
        client = new OkHttpClient();
    }


    protected String get(String url) throws IOException, Exception {

        String credential = Credentials.basic(authenticationToken, "");

        Request request = new Request.Builder().url(url).header("User-Agent", userAgent)
                .header("Authorization", credential).addHeader("Accept", "application/json;").build();

        Response response = client.newCall(request).execute();
        switch (response.code()) {
	        case 200:
	            return response.body().string();
	        case 401:
	        	log.warn("get() - 401 - Access Denied");
	            throw new Exception("Access Denied");
			case 404:
				log.warn("get() - 404 - Not Found");
				throw new Exception("Not Found");
	        default:
	            throw new Exception("get() - Unknown Error - status code: " + response.code());
        }

    }


    /**
     * Query the API: https://rest.cvrapi.dk/v1/dk/company/vatNumber
     *
     * @param vatNumber
     * @return
     * @throws IOException
     */
    protected String getCompanyJson(String vatNumber) throws IOException, Exception {
        String response = get(baseUrl + "/company/" + vatNumber);
        log.debug("getCompanyJson() response: " + response);
        return response;
    }


    /**
     * Use GSON to deserialize JSON into POJO's.
     *
     * @param json
     * @return
     */
    protected Company parseJsonIntoCompany(String json) throws JsonSyntaxException {
    	Gson gson = new Gson();
        Company company = gson.fromJson(json, Company.class);
        return company;
    }


	/**
	 * Lookup company by VAT number
	 *
	 * @param vatNumber
	 * @return
	 */
    public Company getCompanyByVatNumber(String vatNumber) {

        try {
            String json = getCompanyJson(vatNumber);
            return parseJsonIntoCompany(json);
		} catch (JsonSyntaxException e) {
			log.error("Error parsing JSON", e);
            return null;
        } catch (IOException e) {
            log.error("IO Error", e);
            return null;
        } catch (Exception e) {
            log.error("Unknown Error", e);
            return null;
        }

    }

}
